import multiprocessing
import socket
import os

workers = multiprocessing.cpu_count()
bind = '0.0.0.0:8000'
accesslog = "-"


wsock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

def when_ready(server):
    import threading

    def thread_prefork():
        import os
        from django.core.management import call_command
        os.environ.setdefault("DJANGO_SETTINGS_MODULE", "nicechat.settings")
        import django
        django.setup()
        call_command('migrate')

        from django.conf import settings
        if settings.NODE_MODE == settings.NODE_MODE_WORKER:
            from nicechat.utils.arbiter_sync import node_syncing
            print("Worker Syncing...")
            node_syncing()


    def broadcasting():    
        print('Start broadcasting.', os.getpid())
        msg = b'nc_meetyou'
        appips = list(set([ip[-1][0] for ip in socket.getaddrinfo(host=socket.gethostname(), port=None, family=socket.AF_INET)]))
        for ip in appips:
            wsock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
            wsock.bind(("0.0.0.0", 5005))
            wsock.sendto(msg, ("255.255.255.255", 5005))

        os.environ.setdefault("DJANGO_SETTINGS_MODULE", "nicechat.settings")
        import django
        import json
        django.setup()
        from django.conf import settings
        from nicechat import watcher
        wsock.settimeout(2.0)
        while watcher.can_i_run():
            try:
                data, addr = wsock.recvfrom(1024)
            except socket.timeout:
                continue
            print(data, addr)
            if data == msg:
                if settings.NODE_MODE == settings.NODE_MODE_ARBITER:
                    wsock.sendto(b"iamarbiter", (addr[0], 5005))
                if settings.NODE_MODE == settings.NODE_MODE_WORKER:                    
                    hosts = json.dumps(settings.NODE_ARBITER_HOST).encode()
                    wsock.sendto(hosts, (addr[0], 5005))
        print('Stop broadcasting.', os.getpid())


    dthw = threading.Thread(name='thread_prefork', target=thread_prefork)
    dthw.start()
    dthw.join()

    bcth = threading.Thread(name='broadcasting', target=broadcasting)
    bcth.setDaemon(True)
    bcth.start()


def worker_exit(server, worker):
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "nicechat.settings")
    import django
    import platform
    django.setup()
    from django.conf import settings
    from nicechat import watcher

    watcher.apply_stop()
    from nicechat.utils.arbiter import arbiter_request, arbiter_get_hosts, mark_node_status
    data = {
        'event': 'stopworker',
        'node': platform.node(),
        'mode': settings.NODE_MODE,
        'pid': os.getpid(),
    }
    if settings.NODE_MODE == settings.NODE_MODE_ARBITER:
        mark_node_status(data)
    elif settings.NODE_MODE == settings.NODE_MODE_WORKER:
        hosts = arbiter_get_hosts(settings)
        print(hosts)
        arbiter_request(hosts, 'monitor', data)
    print('DOWN', worker.pid, flush=True)



def on_exit(server):
    print('DOWN-ALL',flush=True)
    wsock.close()
    try:
        wsock.shutdown(1)
    except OSError:
        pass
